"use strict";Object.defineProperty(exports,"__esModule",{value:!0});var e,t=require("../../../base/common/types.js"),o=require("../cursorCommon.js"),n=require("./cursorMoveOperations.js"),i=require("./cursorWordOperations.js"),r=require("../core/position.js"),a=require("../core/range.js");exports.CursorMove=void 0,(e=exports.CursorMove||(exports.CursorMove={})).metadata={description:"Move cursor to a logical position in the view",args:[{name:"Cursor move argument object",description:"Property-value pairs that can be passed through this argument:\n\t\t\t\t\t* 'to': A mandatory logical position value providing where to move the cursor.\n\t\t\t\t\t\t```\n\t\t\t\t\t\t'left', 'right', 'up', 'down', 'prevBlankLine', 'nextBlankLine',\n\t\t\t\t\t\t'wrappedLineStart', 'wrappedLineEnd', 'wrappedLineColumnCenter'\n\t\t\t\t\t\t'wrappedLineFirstNonWhitespaceCharacter', 'wrappedLineLastNonWhitespaceCharacter'\n\t\t\t\t\t\t'viewPortTop', 'viewPortCenter', 'viewPortBottom', 'viewPortIfOutside'\n\t\t\t\t\t\t```\n\t\t\t\t\t* 'by': Unit to move. Default is computed based on 'to' value.\n\t\t\t\t\t\t```\n\t\t\t\t\t\t'line', 'wrappedLine', 'character', 'halfLine'\n\t\t\t\t\t\t```\n\t\t\t\t\t* 'value': Number of units to move. Default is '1'.\n\t\t\t\t\t* 'select': If 'true' makes the selection. Default is 'false'.\n\t\t\t\t",constraint:function(e){if(!t.isObject(e))return!1;const o=e;return!(!t.isString(o.to)||!t.isUndefined(o.select)&&!t.isBoolean(o.select)||!t.isUndefined(o.by)&&!t.isString(o.by)||!t.isUndefined(o.value)&&!t.isNumber(o.value))},schema:{type:"object",required:["to"],properties:{to:{type:"string",enum:["left","right","up","down","prevBlankLine","nextBlankLine","wrappedLineStart","wrappedLineEnd","wrappedLineColumnCenter","wrappedLineFirstNonWhitespaceCharacter","wrappedLineLastNonWhitespaceCharacter","viewPortTop","viewPortCenter","viewPortBottom","viewPortIfOutside"]},by:{type:"string",enum:["line","wrappedLine","character","halfLine"]},value:{type:"number",default:1},select:{type:"boolean",default:!1}}}}]},e.RawDirection={Left:"left",Right:"right",Up:"up",Down:"down",PrevBlankLine:"prevBlankLine",NextBlankLine:"nextBlankLine",WrappedLineStart:"wrappedLineStart",WrappedLineFirstNonWhitespaceCharacter:"wrappedLineFirstNonWhitespaceCharacter",WrappedLineColumnCenter:"wrappedLineColumnCenter",WrappedLineEnd:"wrappedLineEnd",WrappedLineLastNonWhitespaceCharacter:"wrappedLineLastNonWhitespaceCharacter",ViewPortTop:"viewPortTop",ViewPortCenter:"viewPortCenter",ViewPortBottom:"viewPortBottom",ViewPortIfOutside:"viewPortIfOutside"},e.RawUnit={Line:"line",WrappedLine:"wrappedLine",Character:"character",HalfLine:"halfLine"},e.parse=function(t){if(!t.to)return null;let o;switch(t.to){case e.RawDirection.Left:o=0;break;case e.RawDirection.Right:o=1;break;case e.RawDirection.Up:o=2;break;case e.RawDirection.Down:o=3;break;case e.RawDirection.PrevBlankLine:o=4;break;case e.RawDirection.NextBlankLine:o=5;break;case e.RawDirection.WrappedLineStart:o=6;break;case e.RawDirection.WrappedLineFirstNonWhitespaceCharacter:o=7;break;case e.RawDirection.WrappedLineColumnCenter:o=8;break;case e.RawDirection.WrappedLineEnd:o=9;break;case e.RawDirection.WrappedLineLastNonWhitespaceCharacter:o=10;break;case e.RawDirection.ViewPortTop:o=11;break;case e.RawDirection.ViewPortBottom:o=13;break;case e.RawDirection.ViewPortCenter:o=12;break;case e.RawDirection.ViewPortIfOutside:o=14;break;default:return null}let n=0;switch(t.by){case e.RawUnit.Line:n=1;break;case e.RawUnit.WrappedLine:n=2;break;case e.RawUnit.Character:n=3;break;case e.RawUnit.HalfLine:n=4}return{direction:o,unit:n,select:!!t.select,value:t.value||1}},exports.CursorMoveCommands=class{static addCursorDown(e,t,i){const r=[];let a=0;for(let s=0,l=t.length;s<l;s++){const l=t[s];r[a++]=new o.CursorState(l.modelState,l.viewState),r[a++]=i?o.CursorState.fromModelState(n.MoveOperations.translateDown(e.cursorConfig,e.model,l.modelState)):o.CursorState.fromViewState(n.MoveOperations.translateDown(e.cursorConfig,e,l.viewState))}return r}static addCursorUp(e,t,i){const r=[];let a=0;for(let s=0,l=t.length;s<l;s++){const l=t[s];r[a++]=new o.CursorState(l.modelState,l.viewState),r[a++]=i?o.CursorState.fromModelState(n.MoveOperations.translateUp(e.cursorConfig,e.model,l.modelState)):o.CursorState.fromViewState(n.MoveOperations.translateUp(e.cursorConfig,e,l.viewState))}return r}static moveToBeginningOfLine(e,t,o){const n=[];for(let i=0,r=t.length;i<r;i++){const r=t[i];n[i]=this._moveToLineStart(e,r,o)}return n}static _moveToLineStart(e,t,o){const n=t.viewState.position.column,i=n===t.modelState.position.column,r=t.viewState.position.lineNumber,a=e.getLineFirstNonWhitespaceColumn(r);return i||n===a?this._moveToLineStartByModel(e,t,o):this._moveToLineStartByView(e,t,o)}static _moveToLineStartByView(e,t,i){return o.CursorState.fromViewState(n.MoveOperations.moveToBeginningOfLine(e.cursorConfig,e,t.viewState,i))}static _moveToLineStartByModel(e,t,i){return o.CursorState.fromModelState(n.MoveOperations.moveToBeginningOfLine(e.cursorConfig,e.model,t.modelState,i))}static moveToEndOfLine(e,t,o,n){const i=[];for(let r=0,a=t.length;r<a;r++){const a=t[r];i[r]=this._moveToLineEnd(e,a,o,n)}return i}static _moveToLineEnd(e,t,o,n){const i=t.viewState.position,r=e.getLineMaxColumn(i.lineNumber),a=i.column===r,s=t.modelState.position,l=e.model.getLineMaxColumn(s.lineNumber),u=r-i.column==l-s.column;return a||u?this._moveToLineEndByModel(e,t,o,n):this._moveToLineEndByView(e,t,o,n)}static _moveToLineEndByView(e,t,i,r){return o.CursorState.fromViewState(n.MoveOperations.moveToEndOfLine(e.cursorConfig,e,t.viewState,i,r))}static _moveToLineEndByModel(e,t,i,r){return o.CursorState.fromModelState(n.MoveOperations.moveToEndOfLine(e.cursorConfig,e.model,t.modelState,i,r))}static expandLineSelection(e,t){const n=[];for(let i=0,s=t.length;i<s;i++){const s=t[i],l=s.modelState.selection.startLineNumber,u=e.model.getLineCount();let m,c=s.modelState.selection.endLineNumber;c===u?m=e.model.getLineMaxColumn(u):(c++,m=1),n[i]=o.CursorState.fromModelState(new o.SingleCursorState(new a.Range(l,1,l,1),0,0,new r.Position(c,m),0))}return n}static moveToBeginningOfBuffer(e,t,i){const r=[];for(let a=0,s=t.length;a<s;a++){const s=t[a];r[a]=o.CursorState.fromModelState(n.MoveOperations.moveToBeginningOfBuffer(e.cursorConfig,e.model,s.modelState,i))}return r}static moveToEndOfBuffer(e,t,i){const r=[];for(let a=0,s=t.length;a<s;a++){const s=t[a];r[a]=o.CursorState.fromModelState(n.MoveOperations.moveToEndOfBuffer(e.cursorConfig,e.model,s.modelState,i))}return r}static selectAll(e,t){const n=e.model.getLineCount(),i=e.model.getLineMaxColumn(n);return o.CursorState.fromModelState(new o.SingleCursorState(new a.Range(1,1,1,1),0,0,new r.Position(n,i),0))}static line(e,t,n,i,s){const l=e.model.validatePosition(i),u=s?e.coordinatesConverter.validateViewPosition(new r.Position(s.lineNumber,s.column),l):e.coordinatesConverter.convertModelPositionToViewPosition(l);if(!n){const t=e.model.getLineCount();let n=l.lineNumber+1,i=1;return n>t&&(n=t,i=e.model.getLineMaxColumn(n)),o.CursorState.fromModelState(new o.SingleCursorState(new a.Range(l.lineNumber,1,n,i),2,0,new r.Position(n,i),0))}const m=t.modelState.selectionStart.getStartPosition().lineNumber;if(l.lineNumber<m)return o.CursorState.fromViewState(t.viewState.move(!0,u.lineNumber,1,0));if(l.lineNumber>m){const n=e.getLineCount();let i=u.lineNumber+1,r=1;return i>n&&(i=n,r=e.getLineMaxColumn(i)),o.CursorState.fromViewState(t.viewState.move(!0,i,r,0))}{const e=t.modelState.selectionStart.getEndPosition();return o.CursorState.fromModelState(t.modelState.move(!0,e.lineNumber,e.column,0))}}static word(e,t,n,r){const a=e.model.validatePosition(r);return o.CursorState.fromModelState(i.WordOperations.word(e.cursorConfig,e.model,t.modelState,n,a))}static cancelSelection(e,t){if(!t.modelState.hasSelection())return new o.CursorState(t.modelState,t.viewState);const n=t.viewState.position.lineNumber,i=t.viewState.position.column;return o.CursorState.fromViewState(new o.SingleCursorState(new a.Range(n,i,n,i),0,0,new r.Position(n,i),0))}static moveTo(e,t,n,i,a){if(n){if(1===t.modelState.selectionStartKind)return this.word(e,t,n,i);if(2===t.modelState.selectionStartKind)return this.line(e,t,n,i,a)}const s=e.model.validatePosition(i),l=a?e.coordinatesConverter.validateViewPosition(new r.Position(a.lineNumber,a.column),s):e.coordinatesConverter.convertModelPositionToViewPosition(s);return o.CursorState.fromViewState(t.viewState.move(n,l.lineNumber,l.column,0))}static simpleMove(e,t,i,r,a,s){switch(i){case 0:return 4===s?this._moveHalfLineLeft(e,t,r):this._moveLeft(e,t,r,a);case 1:return 4===s?this._moveHalfLineRight(e,t,r):this._moveRight(e,t,r,a);case 2:return 2===s?this._moveUpByViewLines(e,t,r,a):this._moveUpByModelLines(e,t,r,a);case 3:return 2===s?this._moveDownByViewLines(e,t,r,a):this._moveDownByModelLines(e,t,r,a);case 4:return 2===s?t.map((t=>o.CursorState.fromViewState(n.MoveOperations.moveToPrevBlankLine(e.cursorConfig,e,t.viewState,r)))):t.map((t=>o.CursorState.fromModelState(n.MoveOperations.moveToPrevBlankLine(e.cursorConfig,e.model,t.modelState,r))));case 5:return 2===s?t.map((t=>o.CursorState.fromViewState(n.MoveOperations.moveToNextBlankLine(e.cursorConfig,e,t.viewState,r)))):t.map((t=>o.CursorState.fromModelState(n.MoveOperations.moveToNextBlankLine(e.cursorConfig,e.model,t.modelState,r))));case 6:return this._moveToViewMinColumn(e,t,r);case 7:return this._moveToViewFirstNonWhitespaceColumn(e,t,r);case 8:return this._moveToViewCenterColumn(e,t,r);case 9:return this._moveToViewMaxColumn(e,t,r);case 10:return this._moveToViewLastNonWhitespaceColumn(e,t,r);default:return null}}static viewportMove(e,t,o,n,i){const r=e.getCompletelyVisibleViewRange(),a=e.coordinatesConverter.convertViewRangeToModelRange(r);switch(o){case 11:{const o=this._firstLineNumberInRange(e.model,a,i),r=e.model.getLineFirstNonWhitespaceColumn(o);return[this._moveToModelPosition(e,t[0],n,o,r)]}case 13:{const o=this._lastLineNumberInRange(e.model,a,i),r=e.model.getLineFirstNonWhitespaceColumn(o);return[this._moveToModelPosition(e,t[0],n,o,r)]}case 12:{const o=Math.round((a.startLineNumber+a.endLineNumber)/2),i=e.model.getLineFirstNonWhitespaceColumn(o);return[this._moveToModelPosition(e,t[0],n,o,i)]}case 14:{const o=[];for(let i=0,a=t.length;i<a;i++){const a=t[i];o[i]=this.findPositionInViewportIfOutside(e,a,r,n)}return o}default:return null}}static findPositionInViewportIfOutside(e,t,i,r){const a=t.viewState.position.lineNumber;if(i.startLineNumber<=a&&a<=i.endLineNumber-1)return new o.CursorState(t.modelState,t.viewState);{let s;s=a>i.endLineNumber-1?i.endLineNumber-1:a<i.startLineNumber?i.startLineNumber:a;const l=n.MoveOperations.vertical(e.cursorConfig,e,a,t.viewState.position.column,t.viewState.leftoverVisibleColumns,s,!1);return o.CursorState.fromViewState(t.viewState.move(r,l.lineNumber,l.column,l.leftoverVisibleColumns))}}static _firstLineNumberInRange(e,t,o){let n=t.startLineNumber;return t.startColumn!==e.getLineMinColumn(n)&&n++,Math.min(t.endLineNumber,n+o-1)}static _lastLineNumberInRange(e,t,o){let n=t.startLineNumber;return t.startColumn!==e.getLineMinColumn(n)&&n++,Math.max(n,t.endLineNumber-o+1)}static _moveLeft(e,t,i,r){return t.map((t=>o.CursorState.fromViewState(n.MoveOperations.moveLeft(e.cursorConfig,e,t.viewState,i,r))))}static _moveHalfLineLeft(e,t,i){const r=[];for(let a=0,s=t.length;a<s;a++){const s=t[a],l=s.viewState.position.lineNumber,u=Math.round(e.getLineLength(l)/2);r[a]=o.CursorState.fromViewState(n.MoveOperations.moveLeft(e.cursorConfig,e,s.viewState,i,u))}return r}static _moveRight(e,t,i,r){return t.map((t=>o.CursorState.fromViewState(n.MoveOperations.moveRight(e.cursorConfig,e,t.viewState,i,r))))}static _moveHalfLineRight(e,t,i){const r=[];for(let a=0,s=t.length;a<s;a++){const s=t[a],l=s.viewState.position.lineNumber,u=Math.round(e.getLineLength(l)/2);r[a]=o.CursorState.fromViewState(n.MoveOperations.moveRight(e.cursorConfig,e,s.viewState,i,u))}return r}static _moveDownByViewLines(e,t,i,r){const a=[];for(let s=0,l=t.length;s<l;s++){const l=t[s];a[s]=o.CursorState.fromViewState(n.MoveOperations.moveDown(e.cursorConfig,e,l.viewState,i,r))}return a}static _moveDownByModelLines(e,t,i,r){const a=[];for(let s=0,l=t.length;s<l;s++){const l=t[s];a[s]=o.CursorState.fromModelState(n.MoveOperations.moveDown(e.cursorConfig,e.model,l.modelState,i,r))}return a}static _moveUpByViewLines(e,t,i,r){const a=[];for(let s=0,l=t.length;s<l;s++){const l=t[s];a[s]=o.CursorState.fromViewState(n.MoveOperations.moveUp(e.cursorConfig,e,l.viewState,i,r))}return a}static _moveUpByModelLines(e,t,i,r){const a=[];for(let s=0,l=t.length;s<l;s++){const l=t[s];a[s]=o.CursorState.fromModelState(n.MoveOperations.moveUp(e.cursorConfig,e.model,l.modelState,i,r))}return a}static _moveToViewPosition(e,t,n,i,r){return o.CursorState.fromViewState(t.viewState.move(n,i,r,0))}static _moveToModelPosition(e,t,n,i,r){return o.CursorState.fromModelState(t.modelState.move(n,i,r,0))}static _moveToViewMinColumn(e,t,o){const n=[];for(let i=0,r=t.length;i<r;i++){const r=t[i],a=r.viewState.position.lineNumber,s=e.getLineMinColumn(a);n[i]=this._moveToViewPosition(e,r,o,a,s)}return n}static _moveToViewFirstNonWhitespaceColumn(e,t,o){const n=[];for(let i=0,r=t.length;i<r;i++){const r=t[i],a=r.viewState.position.lineNumber,s=e.getLineFirstNonWhitespaceColumn(a);n[i]=this._moveToViewPosition(e,r,o,a,s)}return n}static _moveToViewCenterColumn(e,t,o){const n=[];for(let i=0,r=t.length;i<r;i++){const r=t[i],a=r.viewState.position.lineNumber,s=Math.round((e.getLineMaxColumn(a)+e.getLineMinColumn(a))/2);n[i]=this._moveToViewPosition(e,r,o,a,s)}return n}static _moveToViewMaxColumn(e,t,o){const n=[];for(let i=0,r=t.length;i<r;i++){const r=t[i],a=r.viewState.position.lineNumber,s=e.getLineMaxColumn(a);n[i]=this._moveToViewPosition(e,r,o,a,s)}return n}static _moveToViewLastNonWhitespaceColumn(e,t,o){const n=[];for(let i=0,r=t.length;i<r;i++){const r=t[i],a=r.viewState.position.lineNumber,s=e.getLineLastNonWhitespaceColumn(a);n[i]=this._moveToViewPosition(e,r,o,a,s)}return n}};
//# sourceMappingURL=cursorMoveCommands.js.map
